# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-03-12 15:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SampleModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=128)),
                ('category', models.CharField(choices=[('foo', 'Foo'), ('bar', 'Bar')], max_length=128)),
            ],
        ),
    ]
